#!%%%WISH%%% -f

# RCS Id:
#	$Id: tktex.sed,v 2.2.2.1 1997/07/07 01:09:40 ndanger Release $
#

###     TkTeX:  Tk-based GUI for TeX and related tools.
###     Copyright (C) 1995  Norman Danner
### 
###     This program is free software; you can redistribute it and/or modify
###     it under the terms of the GNU General Public License as published by
###     the Free Software Foundation; either version 2 of the License, or
###     (at your option) any later version.
### 
###     This program is distributed in the hope that it will be useful,
###     but WITHOUT ANY WARRANTY; without even the implied warranty of
###     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
###     GNU General Public License for more details.
### 
###     You should have received a copy of the GNU General Public License
###     along with this program; if not, write to the Free Software
###     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

###      e-mail:	ndanner@indiana.edu
###      WWW:		http://www.cs.indiana.edu/hyplan/ndanner/home.html
###			http://www.cs.indiana.edu/hyplan/ndanner/tktex.html

# Set TKTEX(lib_dir) to the directory in which you have put TkTeX's support
# files.

# cmdtrace on [open tktex.debug w]

# set TKTEX(lib_dir) /usr/local/lib/tk/tktex
set TKTEX(lib_dir) %%TKTEX_LIB_DIR%%
set auto_path [linsert $auto_path 0 $TKTEX(lib_dir)]

set TKTEX(tk3) [fna:util_tk3]
set TKTEX(tk4) [fna:util_tk4]

set TKTEX(version) [fna:ver_rcs {$Revision: 2.2.2.1 $} {$State: Release $}]


# ################
# Utility routines
# ################

# tktex:get_option {w o} -- Get the value of the option o of the widget
#  w.

proc tktex:get_option {w o} {
  
  global TKTEX
  
  if {$TKTEX(tk3)} {
    return [lindex [$w configure $o] 4]
  } else {
    return [$w cget $o]
  }
  
}


# tktex:popup {w t}:  This creates a toplevel window $w with title $t, or, if it
#   already exists, raises it.  Useful for things like Options windows,
#   which people sometimes leave open, but get obscured by other windows.

proc tktex:popup {w t} {

    if {[winfo exists $w]} {
    	raise $w
    	return 1
    } {
    	toplevel $w
    	wm title $w "$t"
    	return 0
    }
    
}

# ########
# Defaults
# ########

proc tktex:var-defaults {} {

	global TKTEX
	global env

	set TKTEX(rc_file) "$env(HOME)/.tk/tktex/tktexrc.tcl"
	set TKTEX(rc_file2) "$env(HOME)/.tktexrc"

	set TKTEX(tex_pid) ""
	set TKTEX(base_file) ""
	set TKTEX(src_file) ""
	set TKTEX(tex_dir) "$env(HOME)"

	set TKTEX(view_cmd) "xdvi"
	set TKTEX(print_cmd) "dvips"
	set TKTEX(edit_cmd) "emacs"
	set TKTEX(use_remote_editor) 0
	set TKTEX(remote_editor_interp) ""
	set TKTEX(show_dir) 0
	set TKTEX(show_cmds) 1
	set TKTEX(clear_new) 1
	set TKTEX(printers) "lp"
	set TKTEX(update) 100
	set TKTEX(interactive) 0

	set TKTEX(inp_buf_size) 80
	set TKTEX(error_exp) {\*$|=$|\?$|Enter file name:$}
	set TKTEX(io_mode) stock
	if {[info tclversion] >= "7.5"} {set TKTEX(io_mode) stock-7.5}
	if {[info commands infox] != ""} {set TKTEX(io_mode) tclX}

	set TKTEX(font) [tktex:get_option $TKTEX(text_widget) -font]
	set TKTEX(background) [tktex:get_option $TKTEX(text_widget) -background]
	set TKTEX(input_fg) [tktex:get_option $TKTEX(text_widget) -foreground]
	set TKTEX(output_fg) $TKTEX(input_fg)
	
	if {[string first "TEXEDIT" [array names env]] == -1} \
	  {set env(TEXEDIT) "xterm -e vi %d %s"}
	
	set TKTEX(dvips,print_dest) "lpr"
	set TKTEX(dvips,pages) "allpages"
	set TKTEX(dvips,pgtype) "texpage"
	set TKTEX(dvips,mfeed) 0
	set TKTEX(dvips,reverse) 0
	set TKTEX(dvips,burst_page) 0
	set TKTEX(dvips,xops) ""
	
	AddFormat TeX "Plain TeX" tex {tex_hook}
	set TKTEX(tex_fmt) ""

	AddProcess "BiBTeX" {tktex:io "bibtex -verbose $TKTEX(src_file)" passive}

}


# ###############
# Variable traces
# ###############

trace variable TKTEX w tktex:var-traces

proc tktex:var-traces {name1 name2 op} {

    global TKTEX
        
    if {$name2 == "src_file"} {
      if {$TKTEX(src_file) == ""} {set s "disabled"} {set s "normal"}
      .compile configure -state $s
      .proc configure -state $s
      .vp configure -state $s
    }
    
}


# ##########
# Procedures
# ##########

#########
# FORMATS
#########


# Formats are stored as lists of the form {lbl cmd hook}. lbl is the label
# of the format; this is used the describe the format in the Formats menu
# (e.g., "Plain TeX").  cmd is the unix command that is executed to run TeX
# with this format (e.g., "tex").  hook is the name of a proc to be executed
# any time the format is set to the one corresponding to this list (e.g.,
# "tex_hook").  The lists are stored as the variables TKTEX(fmts,$fmt), where
# $fmt is the (internal) name of the format, the one that is supplied as an
# argument to SetFormat.

proc AddFormat {fmt lbl cmd hook} {

    global TKTEX
    
#   If this format already exists, assume we are redefining it.  So delete
#   the current definition, and remove its entry from the Formats menus.
    if {[string first "fmts,$fmt" [array names TKTEX]] != -1} {
      unset TKTEX(fmts,$fmt)
      .compile.m delete "$lbl"
    }
    
#   Define the format and add its entry to the Formats menu.
    set TKTEX(fmts,$fmt) [list $lbl $cmd $hook]
    .compile.m add command -label "$lbl" -command "SetFormat $fmt"
    
}


proc ListFormats {} {

    global TKTEX
    
    set ret {}
    foreach n [array names TKTEX] {
    	if {[string first "fmts" $n] == 0} \
    	  {lappend ret [lindex [split $n ","] 1]}
    }
    
    return $ret
    
}


proc SetFormat {f} {

    global TKTEX
    
#   If the format doesn't exist, return an error.
    if {[string first "fmts,$f" [array names TKTEX]] == -1} {return -1}
    
#   Set the format, update the label of the Formats menu button, set tex_cmd
#   to the unix command appropriate for this format, and execute the hook, if
#   one is specified and it exists as a proc.

    set TKTEX(tex_fmt) $f
    .compile configure -text "Format <[lindex $TKTEX(fmts,$f) 0]>"
    set TKTEX(tex_cmd) [lindex $TKTEX(fmts,$f) 1]
    if {[lindex $TKTEX(fmts,$f) 2] != "" && [info procs [lindex $TKTEX(fmts,$f) 2]] != ""} {eval [lindex $TKTEX(fmts,$f) 2]}
    
}


# ######################
# Non-standard Processes
# ######################

# AddProcess {lbl proc}:  Add the entry $lbl to the Processing menu, and
# execute the proc $proc when this entry is selected.  This allows users
# (or admins) to add procs for site-specific and local programs that are
# related to TeX; see the docs for more detail about how to use this.

proc AddProcess {lbl proc} {

    global TKTEX
    
    catch {.proc.m delete "$lbl"}
    .proc.m add command -label "$lbl" -command $proc
    
}


# #######################
# Input/Output Procedures
# #######################

proc tktex:update {msec} {
    
  update idletasks
  after $msec tktex:update $msec
  
}


# tktex:io {cmd type} -- Start an i/o session for some command cmd.  "type"
# must be either 'interactive' or 'passive', indicating whether or not the
# command might expect user input (e.g., 'tex file.tex' is interactive,
# and 'xdvi file.dvi' is not).

proc tktex:io {cmd type} {
  
  global TKTEX
  
# Check usage of the command.
  if {"$type" != "passive" && "$type" != "interactive"} {
    error "usage:  tktex:io <cmd> <passive|interactive>"
  }

# Make sure we aren't trying to start up an interactive i/o session when there
# is already one in progress.
  if {"$type" == "interactive" && $TKTEX(interactive)} {
    error "tktex:io:  Interactive process currently running"
  }
  
# Show the working directory and the command if so desired by the user.  Make
# sure that this text is marked readonly.
  set tmp [$TKTEX(text_widget) index insert]
  if {$TKTEX(show_dir)} \
    {$TKTEX(text_widget) insert insert "Working in directory [pwd]\n"}
  if {$TKTEX(show_cmds)} \
      {$TKTEX(text_widget) insert insert "$cmd\n"}
  $TKTEX(text_widget) tag add readonly $tmp insert

# Start the command.
set cmd_info [tktex:exec-cmd-$TKTEX(io_mode) "$cmd"]
set pipe_r [lindex $cmd_info 0]
set pipe_w [lindex $cmd_info 1]

  
# Some bookkeeping; keep track of whether this process is interactive; if it
# is, then also set TKTEX(interactive) to make sure that another interactive
# process is not started before this one finishes.

  if {"$type" == "interactive"} {
    set TKTEX($pipe_r,interactive) 1
    set TKTEX(interactive) 1
    set TKTEX(interactive_pid) $pipe_w
  } else {
    set TKTEX($pipe_r,interactive) 0
    if {$TKTEX(interactive)} {tkwait variable TKTEX(interactive)}
  }
  
  set TKTEX(tex_error) 0

# Catch the output, using 'addinput' if this is tk3.6, 'fileevent' if tk4.x.
  if {$TKTEX(tk3)} {
    addinput -read $pipe_r "tktex:display-output-$TKTEX(io_mode) $pipe_r"
  } else {
    fileevent $pipe_r readable "tktex:display-output-$TKTEX(io_mode) $pipe_r"
  }
  
}


proc tktex:exec-cmd-stock {cmd} {
  
  set cmd "$cmd 2>@ stdout"
  set pid [open "|$cmd" r+]
  return "$pid $pid"
  
}


proc tktex:exec-cmd-stock-7.5 {cmd} {
  
  set cmd "$cmd 2>@ stdout"
  set pid [open "|$cmd" r+]
  fconfigure $pid -blocking 0
  return "$pid $pid"
  
}

proc tktex:exec-cmd-tclX {cmd} {

    global TKTEX
    
    pipe pipe_r pipe_w
    pipe pipe_blr pipe_blw
    
    set argvzero [lindex $cmd 0]
    set arglist [string range $cmd [string first " " $cmd] end]
    
    flush stdout
    flush stderr
    set forkPid [fork]
    
    set TKTEX(tclX,$forkPid,pipes) \
      "$pipe_r $pipe_w $pipe_blr $pipe_blw"
    set TKTEX(tclX,$forkPid,pipe_r) $pipe_r
    
    if {$forkPid == 0} {
      dup $pipe_w stdout
      dup $pipe_w stderr
      dup $pipe_blr stdin
      execl $argvzero $arglist
    }
    fcntl $pipe_r NONBLOCK 1
    signal trap SIGCHLD {
      set fpid [lindex [wait] 0]
      tktex:display-output-tclX [lindex $TKTEX(tclX,$fpid,pipes) 0]
      foreach p $TKTEX(tclX,$fpid,pipes) {close $p}
      if {$TKTEX($TKTEX(tclX,$fpid,pipe_r),interactive)} \
        {unset TKTEX($TKTEX(tclX,$fpid,pipe_r),interactive)}
      if {$TKTEX(interactive)} {set TKTEX(interactive) 0}
    }
    
    return "$pipe_r $pipe_blw"

}


# tktex:display-output-stock:  Stock mechanism for displaying the output
#   of a command; works with tcl-7.4 or tk-3.6+addinput.  Each time this
#   procedure is called for an interactive process, data is read from the 
#   pipe one byte at a time, up to inp_buf_bytes bytes per call.  Each
#   character is checked against a set of "error characters", contained
#   in error_chars.  If one of these characters is read, then the loop
#   is immediately broken out of, and data is read one byte at a time per
#   call until a newline is encountered, at which point up to inp_buf_bytes
#   can be read again.  The idea is that error_chars contains characters
#   like ? and =, which, when TeX is running, generally indicate that TeX
#   is about to stop and ask for input; in such a situation, we only want
#   to read in one byte at a time to prevent blocking.  But if there are
#   no errors in the file, reading one byte per call results in too much
#   overhead, slowing the process down.  So this is basically a compromise,
#   which requires the administrator/user to know which characters come
#   up just before TeX (or any other interactive process) wants input.
#   Non-interactive processes are always read one byte per call, because they
#   rarely have enough output for the overhead to be a problem, and some
#   (like xdvi) don't terminate, which would result in a blocking read.

proc tktex:display-output-stock {pid} {
  
  global TKTEX
    
  if {$TKTEX($pid,interactive) || !$TKTEX(interactive)} {
    if {![eof $pid]} {
      if {$TKTEX(tex_error) || !$TKTEX($pid,interactive)} {
        set c [read $pid 1]
        $TKTEX(text_widget) insert insert $c
        $TKTEX(text_widget) tag add readonly insert-1char
        $TKTEX(text_widget) yview -pickplace insert
        if {"$c" == "\n"} {
          set TKTEX(tex_error) 0
          update idletasks
        }
      } else {
        for {set i 0} {$i < $TKTEX(inp_buf_size)} {incr i} {
          set c [read $pid 1]
          append buf $c
            if {[regexp -- $TKTEX(error_exp) $buf]} {
            set TKTEX(tex_error) 1
            break
          }
        }
        set tmp [$TKTEX(text_widget) index insert]
        $TKTEX(text_widget) insert insert $buf
        $TKTEX(text_widget) tag add readonly $tmp insert
        $TKTEX(text_widget) yview -pickplace insert
        if {[string first "\n" $buf] != -1} {update idletasks}
      }
    } {
      if {$TKTEX(tk3)} {removeinput $pid}
      catch "close $pid"
      if {$TKTEX($pid,interactive)} {unset TKTEX($pid,interactive)}
      if {$TKTEX(interactive)} {set TKTEX(interactive) 0}
    }  
  }
  
}


# tktex:display-output-stock-7.5:  At last, we have non-blocking i/o
#  built into the kernel.  Read whatever is there and print it out.
#  Make sure to update at least every newline.

proc tktex:display-output-stock-7.5 {pid} {
  
  global TKTEX
  global errorCode
  
  if {$TKTEX($pid,interactive) || !$TKTEX(interactive)} {
      set buf [read $pid]
      set tmp [$TKTEX(text_widget) index insert]
      $TKTEX(text_widget) insert insert $buf
      $TKTEX(text_widget) tag add readonly $tmp insert
      $TKTEX(text_widget) yview -pickplace insert
      if {[string first "\n" $buf] != -1} {update idletasks}
  }
  if {[eof $pid]} {
    catch "close $pid"
    if {$TKTEX($pid,interactive)} {unset TKTEX($pid,interactive)}
    if {$TKTEX(interactive)} {set TKTEX(interactive) 0}
  }
  
}

# tktex:display-output-tclX:  This isn't much different than stock 7.5.
#  However, because of the way the process has to be executed, the end of
#  the process has to be taken care of by trapping a signal and going
#  through a handler; this is taken care of in tktex:exec-cmd-tclX.

proc tktex:display-output-tclX {pid} {
  
  global TKTEX
  global errorCode
  
  if {$TKTEX($pid,interactive) || !$TKTEX(interactive)} {
      set buf [read $pid]
      set tmp [$TKTEX(text_widget) index insert]
      $TKTEX(text_widget) insert insert $buf
      $TKTEX(text_widget) tag add readonly $tmp insert
      $TKTEX(text_widget) yview -pickplace insert
      if {[string first "\n" $buf] != -1} {update idletasks}
  }
  
}


# ##################
# Editing Procedures
# ##################

# edit-file {f}:  Edit the file $f.  Start the editor if either we are
#  not communicating with a remote (tk) editor, or the interpreter for
#  that editor no longer exists.  Otherwise tell the editor to open the
#  file in a new window.

proc tktex:edit-file {f} {
  global TKTEX
  
  if {!$TKTEX(use_remote_editor) || \
      [lsearch -exact [winfo interps] $TKTEX(remote_editor_interp)] == -1} {
    tktex:io "$TKTEX(edit_cmd) $f" passive
  } {
    send $TKTEX(remote_editor_interp) [format $TKTEX(remote_editor_newfile) $f]
  }
}


# editor-interp {i}:  Set the name of the interpreter for the remote (tk)
#  editor. This will generally be executed by the editor process with the
#  'send' command, so that editor will need to know this interpreter's
#  name.

proc tktex:editor-interp {i} {
  global TKTEX
  set TKTEX(remote_editor_interp) $i
}

# editor-new-file {f}:  The command to send to the remote editor to open
#  a window with a file.  A '%s' in the command will be replaced by the
#  file name.  This proc can be called either from the editor with 'send'
#  or from a startup file.

proc tktex:editor-new-file {f} {
  global TKTEX
  set TKTEX(remote_editor_newfile) $f
}


# ##################
# Viewing Procedures
# ##################

# view-file {file fmt}:  View $file using the procedure tktex:view-$fmt.
#  Except for $fmt=="man", verify that the file exists and that the format
#  is supported; if not, display an error.

proc tktex:view-file {file fmt} {
  
  global TKTEX
  
  if {$fmt != "man" && ![file exists $file]} {
    tk_dialog .tktex_view_file "File Missing" \
      "The file $file does not exist; please notify your system administrator about this." "" 0 "OK"
    return
  }
  
  if {[info commands tktex:view-$fmt] == ""} {
    tk_dialog .tktex_view_file "Bad Format" \
      "The format $fmt is not supported; please notify your system administrator about this." "" 0 "OK"
    return
  } 
  
  tktex:view-$fmt $file
  
  return
  
}


#
# A few simple file viewing procs as defaults.
#

proc tktex:view-dvi {file} {
  global TKTEX
  tktex:io "$TKTEX(view_cmd) $file" passive
}
proc tktex:view-info {file} {
  tktex:io "xterm -e info -f $file" passive
}
proc tktex:view-man {file} {
  tktex:io "xterm -e man $file" passive
}
proc tktex:view-ps {file} {
  tktex:io "ghostview $file" passive
}
proc tktex:view-text {file} {
  toplevel .tktex_view_text
  wm title .tktex_view_text "View Text File"
  ScrollText .tktex_view_text.st
  set t [.tktex_view_text.st text]
  set fId [open $file]
  while {![eof $fId]} {
    $t insert insert "[gets $fId]\n"
  }
  $t configure -state disabled
  close $fId
  button .tktex_view_text.close -text "Close" \
    -command {destroy .tktex_view_text}
  pack .tktex_view_text.st .tktex_view_text.close
}


# ###################
# Printing Procedures
# ###################

# tktex:build-dvips-cmd:  Creates the command to be executed when DVIPS is the
#   selected DVI printing program and "Print" is selected.

proc tktex:build-dvips-cmd {f} {

  global TKTEX
  
  set cmd "$TKTEX(print_cmd)"
  
  if {$TKTEX(dvips,print_dest) == "lpr"} {
    if {$TKTEX(dvips,burst_page)} \
      {append cmd " -o \"\\!lpr -P$TKTEX(printer_name)\""} \
      {append cmd " -o \"\\!lpr -P$TKTEX(printer_name) -h\""}
  } {
    append cmd " -o $TKTEX(dvips,file_name)"
  }
  
  if {$TKTEX(dvips,pages) == "somepages"} {
    if {$TKTEX(dvips,spage) != ""} {
      if {$TKTEX(dvips,pgtype) == "abspage"} \
        {append cmd " -p \"=$TKTEX(dvips,spage)\""} \
	{append cmd " -p $TKTEX(dvips,spage)"}
    }
    if {$TKTEX(dvips,epage) != ""} {
      if {$TKTEX(dvips,pgtype) == "abspage"} \
        {append cmd " -l \"=$TKTEX(dvips,epage)\""} \
      	{append cmd " -l $TKTEX(dvips,epage)"}
    }
  }
  
  if {$TKTEX(dvips,mfeed)} {append cmd " -m"}
  if {$TKTEX(dvips,reverse)} {append cmd " -r"}
  if {$TKTEX(dvips,xops) != ""} {append cmd " $TKTEX(dvips,xops)"}
  
  append cmd " $f"
  return $cmd
  
}


# tktex:dvips-options:  Options panel for dvips.  The theory is that there could
# also be a 'dvi2ps-options', 'dvi2im72-options', or whatever, depending on
# what printing program is being used.

proc tktex:dvips-options {} {
    global TKTEX
    
    if {[tktex:popup .dvips_ops "DVIPS Options"]} {return}
    set f .dvips_ops
        
    frame $f.dest -relief raised -bd 1
    frame $f.dest.f1
    frame $f.dest.l
    frame $f.dest.r
    
    label $f.dest.lbl -text "Printer Options"
    
    radiobutton $f.dest.file -text "File" -variable TKTEX(dvips,print_dest) \
      -value "file"
    fna:bw_labelentry $f.dest.file_name "File Name:" hor \
      {-textvar TKTEX(dvips,file_name)}
    radiobutton $f.dest.lpr -text "Printer" -variable TKTEX(dvips,print_dest) \
      -value "lpr"
    fna:bw_mbentry $f.dest.lpr_name "Printer name:" hor \
      {-textvar TKTEX(printer_name)}
    if {$TKTEX(tk4)} {[$f.dest.lpr_name menu] configure -tearoff 0}
    foreach p $TKTEX(printers) {
      [$f.dest.lpr_name menu] add command -label $p \
        -command "set TKTEX(printer_name) $p"
    }
    
    pack $f.dest.file $f.dest.lpr -in $f.dest.l -fill x
    pack $f.dest.file_name $f.dest.lpr_name -in $f.dest.r -fill x -expand 1
    pack $f.dest.l $f.dest.r -in $f.dest.f1 -side left
    pack configure $f.dest.r -expand 1
    pack $f.dest.lbl $f.dest.f1 -fill x
    
    frame $f.pg -relief raised -bd 1
    frame $f.pg.f1
    frame $f.pg.r
    frame $f.pg.l
    
    label $f.pg.lbl -text "Page Range"
    
    radiobutton $f.pg.allpages -text "All pages" -variable TKTEX(dvips,pages) \
      -value "allpages" -command "
      $f.pg.abspage configure -state disabled
      $f.pg.texpage configure -state disabled
    "
    radiobutton $f.pg.somepages -text "Some pages" \
      -variable TKTEX(dvips,pages) -value "somepages" -command "
      $f.pg.abspage configure -state normal
      $f.pg.texpage configure -state normal
    "

   radiobutton $f.pg.abspage -text "Absolute" -variable TKTEX(dvips,pgtype) \
     -value "abspage"
   radiobutton $f.pg.texpage -text "TeX pages" -variable TKTEX(dvips,pgtype) \
     -value "texpage"
   
   if {$TKTEX(dvips,pages) == "allpages"} "$f.pg.allpages invoke"

   fna:bw_labelentry $f.pg.spage "Start:" hor {-textvar TKTEX(dvips,spage)}
   fna:bw_labelentry $f.pg.epage "End:" hor {-textvar TKTEX(dvips,epage)}
    
    pack $f.pg.allpages $f.pg.abspage $f.pg.spage -in $f.pg.l -fill x
    pack $f.pg.somepages $f.pg.texpage $f.pg.epage -in $f.pg.r -fill x
    pack $f.pg.l $f.pg.r -in $f.pg.f1 -side left
    pack $f.pg.lbl $f.pg.f1
    
    frame $f.misc -relief raised -bd 1
    label $f.misc.lbl -text "Miscellaneous Options"
    
    checkbutton $f.misc.mfeed -text "Manual feed" -variable TKTEX(dvips,mfeed)
    checkbutton $f.misc.rev -text "Reverse pages" -variable TKTEX(dvips,reverse)
    checkbutton $f.misc.burst -text "Burst page" -variable TKTEX(dvips,burst_page)
    
    fna:bw_labelentry $f.misc.xops "Other options:" hor \
      {-textvar TKTEX(dvips,xops) -width 35}
    
    fna:cw_combowin $f.misc \
      $f.misc.lbl \
      "$f.misc.mfeed $f.misc.rev $f.misc.burst" \
      $f.misc.xops
    
    button $f.dis -text "Dismiss" -command "destroy $f"
        
    fna:cw_combowin $f \
      $f.dest \
      $f.pg \
      $f.misc \
      $f.dis
}


#
# tktex:set-options:  Helper applications, environment.
#
proc tktex:set-options {} {

    global TKTEX
    
    if {[tktex:popup .optionset "TkTeX Options"]} {return}
        
    frame .optionset.f1 -relief flat
    frame .optionset.f2 -relief flat
    
    frame .optionset.f1v1 -relief raised -bd 1
    label .optionset.helpers -text "Helper Applications"
    fna:bw_labelentry .optionset.viewer "DVI previewer:" hor \
      {-textvar TKTEX(view_cmd)}
    fna:bw_labelentry .optionset.editor "Editor:" hor {-textvar TKTEX(edit_cmd)}
    if {$TKTEX(use_remote_editor)} \
      {[.optionset.editor entry] configure -textvar TKTEX(remote_edit_cmd)}
    checkbutton .optionset.remote_editor -text "Remote Editor" \
      -variable TKTEX(use_remote_editor) \
      -command {
        if {$TKTEX(use_remote_editor)} {[.optionset.editor entry] configure -textvar TKTEX(remote_edit_cmd)} {[.optionset.editor entry] configure -textvar TKTEX(edit_cmd)}
      }
    pack .optionset.helpers .optionset.viewer .optionset.editor \
      .optionset.remote_editor -in .optionset.f1v1 -side top -fill x
    
    frame .optionset.f1v2 -relief raised -bd 1
    label .optionset.misc -text "Miscellaneous"
    checkbutton .optionset.show_dir -text "Show working directory" \
      -variable TKTEX(show_dir)
    checkbutton .optionset.show_cmds -text "Show commands" \
      -variable TKTEX(show_cmds)
    checkbutton .optionset.clear_on_new -text "Clear on new file" \
      -variable TKTEX(clear_new)
    pack .optionset.misc .optionset.show_dir .optionset.show_cmds .optionset.clear_on_new -in .optionset.f1v2 -side top -fill x

    pack .optionset.f1v1 .optionset.f1v2 -in .optionset.f1 -side top -fill both
    
    frame .optionset.f2v1 -relief raised -bd 1
    label .optionset.env -text "Environment"
    fna:bw_labelentry .optionset.texinputs "TEXINPUTS:" hor \
      {-textvar env(TEXINPUTS) -width 30}
    fna:bw_labelentry .optionset.bibinputs "BIBINPUTS:" hor \
      {-textvar env(BIBINPUTS) -width 30}
    fna:bw_labelentry .optionset.texedit "TEXEDIT:" hor \
      {-textvar env(TEXEDIT) -width 30}
    fna:bw_labelentry .optionset.display "DISPLAY:" hor \
      {-textvar env(DISPLAY) -width 30}
    pack .optionset.env .optionset.texinputs .optionset.bibinputs .optionset.texedit .optionset.display \
      -in .optionset.f2v1 -side top -fill x
    
    pack .optionset.f2v1 -in .optionset.f2 -side top -fill both
   
    button .optionset.dis -text "Dismiss" -command {destroy .optionset}
    pack .optionset.dis -side bottom -fill x
    pack .optionset.f1 .optionset.f2 -side left -fill both -expand yes
    
}


# ##############################
# Make all the menus and buttons
# ##############################

#
# tktex:mbdefault:  Clicking mouse button 3 over menubutton $mb will invoke
#	      index $index of $menu.
#
proc tktex:mbdefault {mb menu index} {
    bind $mb <ButtonRelease-3> "
     if {\"\[tktex:get_option %W -state\]\" == \"disabled\"} {return} ;
     $menu invoke \"$index\"
    "
}


#
# about_window:  Display an info window
#

proc tktex:about {} {
  global TKTEX
  tktex:popup .about "About TkTeX"
  
  label .about.l -text "TkTeX v$TKTEX(version)" -font "*-helvetica-bold-r-normal--24-*"
  message .about.desc -aspect 600 \
    -text {TkTeX:  Tk-based GUI for TeX and related tools.}
  message .about.disc -aspect 600 -text {\
TkTeX, Copyright (C) 1995 Norman Danner.\
TkTeX comes with ABSOLUTELY NO WARRANTY.\
This is free software, and you are welcome to redistribute it\
under the conditions of the GNU Public License Version 2.\
  }
  message .about.address -aspect 1000 -font "8x13" -text {\
ndanner@indiana.edu
http://www.cs.indiana.edu/hyplan/ndanner/home.html\
http://www.cs.indiana.edu/hyplan/ndanner/tktex.html
  }
  button .about.b -text "Dismiss" -command {destroy .about}
  
  pack .about.l .about.desc .about.disc .about.address .about.b -in .about
}


proc tktex:convert-filename {f} {
  set f [glob $f]
  if {[string first "./" $f] == 0} {set f "[pwd]/[string range $f 2 end]"}
  if {[string first "/" $f] != 0} {set f "[pwd]/$f"}
  return $f
}


# tktex:new-file {f} -- Open a new source file.  If $f == "", prompt for
# a file.
#	base_file:	The full pathname of the selected file.
#	src_file:	The rootname of the the selected file (i.e., base_file
#			with the path and extension deleted).
#	tex_file:	The name of the file selected (i.e., base_file with the
#			path deleted).  We don't assume TeX files necessarily
#			use the extension .tex, since some people like .ltx,
#			.dtx, etc.
#	dvi_file:	src_file.dvi.

proc tktex:new-file {f} {

    global TKTEX
      
    set tmp $TKTEX(base_file)
    if {($f == "") || ![file exists $f]} {
      set TKTEX(base_file) \
        [fna:cw_fileselect .fileSelect $TKTEX(tex_dir) "Select Input File" OK \
         "*.tex"]
    } else {
      set TKTEX(base_file) [tktex:convert-filename $f]
    }
    if {$TKTEX(base_file) != "" && ![file exists $TKTEX(base_file)]} {
      set ret_val [tk_dialog .newfile "Create a File" \
        "The file $TKTEX(base_file) does not exist.  Create it?" \
        "" 0 "OK" "Cancel"]
      switch $ret_val \
        0	"exec touch $TKTEX(base_file)" \
        1	{set TKTEX(base_file) ""}
    }
    if {$TKTEX(base_file) != ""} {
      tktex:new-dir [file dir $TKTEX(base_file)]
      set TKTEX(src_file) [file rootname [file tail $TKTEX(base_file)]]
      set TKTEX(tex_file) [file tail $TKTEX(base_file)]
      set TKTEX(dvi_file) "$TKTEX(src_file).dvi"
      tktex:build-edit-menu "$TKTEX(tex_dir)/$TKTEX(tex_file)"
      set TKTEX(file_lbl) "File <$TKTEX(tex_file)>"
      cd $TKTEX(tex_dir)
      if {$TKTEX(clear_new)} {$TKTEX(text_widget) delete 1.0 end}
      set TKTEX(dvips,file_name) "$TKTEX(src_file).ps"
    } {
      set TKTEX(base_file) $tmp
    }

}


# tktex:new-dir {d} : Set the new directory.   First check to see if the
#  directory as given is valid, and, if so, then glob, handle ./ and
#  relative directory references.

proc tktex:new-dir {d} {
  
  global TKTEX
  if {![file isdirectory $d]} {return}
  set TKTEX(tex_dir) "[tktex:convert-filename $d]"
  
}


#
# tktex:build-edit-menu:  Given a base file, find all the files \include'd in
# it, and put them in the edit menu.
#
proc tktex:build-edit-menu {f} {
  
  global TKTEX

#  Don't delete the "New File" entry.  The "perforation"
#  counts as a menu item in tk4.x.  The separator entry seems to get deleted
#  and immediately added because when this menu is empty, the separator
#  doesn't seem to count as an item; in particular, if we just leave it
#  alone, the first time a file is selected, it will get deleted, and then
#  not added back.
  if {$TKTEX(tk3)} {.file.m delete 1 last} {.file.m delete 2 last}
  .file.m add separator
  
# Visually isolate the main (galley) file.
  set dir [file dir $f]
  set main [file tail $f]
  .file.m add command -label "$main" -command \
    "tktex:edit-file $f"
#     "tktex:io \"\$TKTEX(edit_cmd) $f\" passive"
#     "eval \"exec \$TKTEX(edit_cmd) $f &\""
  .file.m add separator
  
  set fId [open $f r]
  
# flist becomes a list of filenames that occur inside \include statements
# (have to make sure we aren't looking at an \includeonly for this) and
# \input statements.
  set flist {}
  while {[gets $fId line] >= 0} {
    if {[string first "\\include" $line] != -1 && \
	[string first "\\includeonly" $line] == -1} {
	    lappend flist "[lindex [split $line "\{\}"] 1]"
    }
    if {[string first "\\input" $line] == 0} {
    	lappend flist "[lindex [split $line " "] 1]"
    }
  }
  close $fId
  
# Create the menu consisting of files in flist.  If a file doesn't have an
# extension, assume it is ".tex".  Add the file to the menu only if it exists
# in the same directory as the main file.  This makes the assumption that files
# that are included but in different directories are macro files or some such,
# and hence not something we would want to both with finding and editing very
# often.
  foreach f $flist {
    if {[file extension $f] == ""} {set f "$f.tex"}
    if {![file exists $dir/$f]} continue
    .file.m add command -label $f -command \
      "tktex:edit-file $dir/$f"
#       "tktex:io \"\$TKTEX(edit_cmd) $dir/$f\" passive"
#       "eval \"exec \$TKTEX(edit_cmd) $dir/$f &\""
  }
  
}


#
# build-help-menu:  Parse TKTEX(doc_menu) and create menu of local help
#  files.  Prepend tktex documentation to the menu.  TKTEX(doc_menu) is
#  a proper Tcl list of menu items; see the tktex:help-menu-item proc for
#  the possible forms for a menu item.
#

proc tktex:build-help-menu {menu} {
  
  global TKTEX
  
  set gen [subst {{TkTeX Manual} {$TKTEX(lib_dir)/tktex-gen.dvi} {dvi}}]
  set var \
    [subst {{TkTeX Variables} {$TKTEX(lib_dir)/tktex-vars.dvi} {dvi}}]
  
  if {[info exists TKTEX(doc_menu)]} {
    set TKTEX(doc_menu) [linsert $TKTEX(doc_menu) 0 $gen $var "-"]
  } {
    set TKTEX(doc_menu) [list $gen $var]
  }

  foreach d $TKTEX(doc_menu) {tktex:help-menu-item $menu $d}
  
}


#
# tktex:help-menu-item {menu item}:  Add an item to the help menu.  A menu
#  item can have any of the following forms:
#    {-}:  A separator
#    {{Label} {File} {Type}}:  This adds a menu item with label Label.
#      Selecting this item invokes tktex:view-file $File $Type.
#    {CascadeName MenuItemList}:  This adds a submenu with label CascadeName.
#      tktex:help-menu-item is invoked for each item in MenuItemList.  The
#      menu for the submenu is created automatically.
#

proc tktex:help-menu-item {menu item} {
  if {$item == "-"} {
    $menu add separator
    return
  }
  if {[llength $item] == 2} {
    set i 0
    while {[winfo exists $menu.$i]} {incr i}
    menu $menu.$i
    $menu add cascade -menu $menu.$i -label [lindex $item 0]
    foreach d [lindex $item 1] {tktex:help-menu-item $menu.$i $d}
    return
  }
  $menu add command -label [lindex $item 0] -command \
    "tktex:view-file [lindex $item 1] [lindex $item 2]"
}


# tktex:make-menus:  Make the various menu-type buttons.

proc tktex:make-menus {mbar} {

  global TKTEX

# .file:  Select a base file and build the File menu.  Here's what all the
#    variables are:
#
  menubutton .file -textvar TKTEX(file_lbl) -menu .file.m
  set TKTEX(file_lbl) "File <>"
  menu .file.m
  tktex:mbdefault .file .file.m "New File"
  .file.m add command -label "New File" -command {tktex:new-file ""}
  .file.m add separator

# .compile:  Select and run a TeX format.
  menubutton .compile -menu .compile.m
  menu .compile.m
  tktex:mbdefault .compile .compile.m "Run TeX"
  .compile.m add command -label "Run TeX" -command {
    tktex:io "$TKTEX(tex_cmd) $TKTEX(tex_file)" interactive
  }
  .compile.m add separator

# .vp:  View and print.
  menubutton .vp -menu .vp.m -text "View/Print"
  menu .vp.m
  tktex:mbdefault .vp .vp.m "View"
  .vp.m add command -label "View" -command {
    tktex:view-file $TKTEX(dvi_file) dvi
  }
  .vp.m add command -label "Print" -command {
    tktex:io [tktex:build-$TKTEX(print_cmd)-cmd $TKTEX(dvi_file)] passive
  }
  .vp.m add separator
  .vp.m add command -label "Print options" \
    -command {tktex:$TKTEX(print_cmd)-options}

# .proc:  Miscellaneous (site-dependent) processing (e.g., bibtex, makeindex)
  menubutton .proc -menu .proc.m -text "Processing"
  menu .proc.m

# .help:  Show help files, available format information.
  menubutton .help -menu .help.m -text "Help"
  menu .help.m
  .help.m add command -label "About" -command {tktex:about}
  .help.m add separator
  .help.m add command -label "Show Formats" -command {
      set TKTEX(tmp) [$TKTEX(text_widget) index insert]
      $TKTEX(text_widget) insert end "---------------------------------------\n"
      $TKTEX(text_widget) insert end "           AVAILABLE FORMATS           \n"
      $TKTEX(text_widget) insert end "---------------------------------------\n"
      foreach f [ListFormats] {
  	$TKTEX(text_widget) insert end "$f:\n"
  	$TKTEX(text_widget) insert end "\tLabel:  [lindex $TKTEX(fmts,$f) 0]\n\tCommand:  [lindex $TKTEX(fmts,$f) 1]\n\tHook:  [lindex $TKTEX(fmts,$f) 2]\n"
      }
      $TKTEX(text_widget) tag add readonly $TKTEX(tmp) insert
      $TKTEX(text_widget) yview -pickplace insert
  }
  pack .file .compile .vp .proc .help -in $mbar -side left -padx 5

}


# tktex:make-buttons {mbar}:  Make button-type objects and pack into mbar

proc tktex:make-buttons {mbar} {
  
  global TKTEX
  
  button .options -text "Options" -command {tktex:set-options}
  
  button .clear -text "Clear" -command {
      $TKTEX(text_widget) delete 1.0 end
      update
  }
  
  button .quit -text "Quit" -command {exit}
  
  pack .options .clear .quit \
    -in $mbar -side left -fill x -expand yes
  
}


# tktex:make-text:  Make the text widget.

proc tktex:make-text {t} {
  
  global TKTEX
  
  fna:bw_scrolltext $t {-setgrid 1}
  
}


# tktex:bind-text:  Bindings for the text widget.

proc tktex:bind-text {} {
  
  global TKTEX
  
  if {$TKTEX(tk3)} {
    bind $TKTEX(text_widget) <Return> {
      if {!$TKTEX(interactive)} {return}
      puts $TKTEX(interactive_pid) \
        [$TKTEX(text_widget) get [fna_int:ro_prevreadonly %W insert]+1c insert]
      flush $TKTEX(interactive_pid)
      if {[fna_int:ro_readonlytag %W insert]} return
      %W insert insert \n; %W yview -pickplace insert
    }
  } else {
    bind $TKTEX(text_widget) <Return> {
      if {!$TKTEX(interactive)} {return}
      puts $TKTEX(interactive_pid) \
        [$TKTEX(text_widget) get [fna_int:ro_prevreadonly %W insert]+1c insert]
      flush $TKTEX(interactive_pid)
    }
  }
  
}


# tktex:bind-entry:  Bindings for entries.  This is only necessary for tk3.x,
#   since tk4.x has reasonable defaults for entry widgets.

proc tktex:bind-entry {} {
  
  global TKTEX
  
# Get arrow keys in Entry widgets to do something useful--scrolling with
# the middle mousebutton doesn't seem to occur naturally in nature.

  if {$TKTEX(tk3)} {
    bind Entry <Left> {
      %W icursor [expr [%W index insert]-1]
      %W view [expr "[%W index insert]-([lindex [%W configure -width] 4]/2)"]
    }
    bind Entry <Right> {
      %W icursor [expr [%W index insert]+1]
      %W view [expr "[%W index insert]-([lindex [%W configure -width] 4]/2)"]
    }
  }
  
}

# #####################
# Build the main window
# #####################

frame .menubar
tktex:make-menus .menubar
tktex:make-buttons .menubar
tktex:make-text .text
set TKTEX(text_widget) [.text text]
tktex:bind-text
tktex:bind-entry

pack .menubar .text -side top -fill both -expand yes
focus $TKTEX(text_widget)


# #######################
# Set a bunch of defaults
# #######################

# Widget set
fna:cw_init

# Read-only tags for text
fna:ro

# TkTeX variable defaults
tktex:var-defaults


# ########################################
# Parse the argument list, if there is any
# ########################################

set TKTEX(setfile) 0
set TKTEX(setdir) 0

for {set i 0} {$i < $argc} {incr i 2} {
  set val [lindex $argv [expr $i+1]]
  
  switch -- [lindex $argv $i] \
    -dir	"set TKTEX(setdir) $val" \
    -file	"set TKTEX(setfile) $val" \
    -io_mode	"set TKTEX(io_mode) $val" \
    -rcfile	"set TKTEX(rc_file) [tktex:convert-filename $val]" \
    default	"puts \"Ignoring unknown option [lindex $argv $i]\""

}

# Read in .rc file, if it exists
if {[file exists $TKTEX(lib_dir)/tktex.rc]} {source $TKTEX(lib_dir)/tktex.rc}
if {[file exists $TKTEX(rc_file)]} {
  source $TKTEX(rc_file)
} {
  if {[file exists $TKTEX(rc_file2)]} {source $TKTEX(rc_file2)}
}

if {$TKTEX(setdir) != 0} {tktex:new-dir $TKTEX(setdir)}
if {$TKTEX(setfile) != 0} {tktex:new-file $TKTEX(setfile)}
unset TKTEX(setdir)
unset TKTEX(setfile)

tktex:build-help-menu {.help.m}

if {$TKTEX(tex_fmt) == ""} {SetFormat TeX}
set TKTEX(printer_name) [lindex $TKTEX(printers) 0]
$TKTEX(text_widget) configure -background $TKTEX(background)
$TKTEX(text_widget) configure -foreground $TKTEX(input_fg)
$TKTEX(text_widget) tag configure readonly -foreground $TKTEX(output_fg)
$TKTEX(text_widget) configure -insertbackground $TKTEX(input_fg)
$TKTEX(text_widget) configure -font $TKTEX(font)

wm title . "TkTeX v$TKTEX(version)"
